unit WicFileSave;

interface

uses
  ActiveX, WinCodec, Classes, Graphics;

type
  TWICImageHelper = class helper for TWICImage
  public
    procedure FixedSaveTofile(const Filename: string);
    procedure FixedSaveToStream(Stream: TStream);
  end;

implementation

{ TWICImageHelper }

procedure TWICImageHelper.FixedSaveTofile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmCreate);
  try
    FixedSaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TWICImageHelper.FixedSaveToStream(Stream: TStream);
var
  FGData: TMemoryStream;
  Encoder: IWICBitmapEncoder;
  Frame: IWICBitmapFrameEncode;
  Props: IPropertyBag2;
  LStreamAdapter: IStream;
  PixelFormat: TGUID;
  LStream: IWICStream;
  Palette: IWICPalette;
begin
  FGData := TMemoryStream.Create;
  try
    LStreamAdapter := TStreamAdapter.Create(FGData);

    ImagingFactory.CreateStream(LStream);
    LStream.InitializeFromIStream(LStreamAdapter);
    ImagingFactory.CreateEncoder(EncoderContainerFormat, guid_null, Encoder);

    Encoder.Initialize(LStream, WICBitmapEncoderNoCache);
    Encoder.CreateNewFrame(Frame, Props);

    Frame.Initialize(Props);
    Handle.GetPixelFormat(PixelFormat);
    Frame.SetPixelFormat(PixelFormat);

    Frame.SetSize(Width, Height);

    ImagingFactory.CreatePalette(Palette);
    Handle.CopyPalette(Palette);
    Frame.SetPalette(Palette);
    Frame.WriteSource(Handle, nil);
    Frame.Commit;
    Encoder.Commit;

    FGData.Position := 0;
    Stream.CopyFrom(FGData, FGData.Size);
  finally
    FGData.Free;
  end;
end;

end.

